/*
 * DbApplication.java
 *
 * Created on 28. Mai 2001, 14:51 by Christoph Knabe
 */

package bank3tier.db;

import bank3tier.db.Pool.NoPersistentClassExc;
import bank3tier.ut.StringUtil;
import multex.Failure;


/**Here the database access layer is composed.
 * The persistence manager is elected, as well as the necessary database tables.
 * This DbApplication uses _JoraPool as persistence layer and the tables
 * Kunde, Konto, Verwaltet. 
* @author  2005-03-29  Christoph Knabe
*/
public class DbApplication {

    /**The object of the persistence manager*/
    /*package*/ _JoraPool _pool;
    
    /** Creates new DbBankApplication */
    public DbApplication() {
        try{
            _pool = new _JoraPool();
            //bank3tier.db.TestAll.setUpEmptyDB();
            //DbObject.dbOpenDB(this);
            //_openDB();
		    _createTablesBySql(DbObject.oidSuffix);
		    dbEndTransaction(true);
		}catch(final Exception ex){
		    throw new multex.Failure("Failure creating the application "
		        + "specific database", ex
		    );
		}
    }

     /**Creates an empty instance of the application database without opening it
    public abstract void createEmptyDB();
    */

    /**Opens an application specific JORA database.*/
    //private void openDB(){}

    /**Returns the JORA session used to access the application database.
       Requires: openDB() was successfully executed before.
       @deprecated
    */
    private jora.Session _getSession() {return _pool.getSession();}

    /**Creates an empty instance of the Bank3Tier database without opening it
    public void createEmptyDB(){
        final String emptyDbFileName = "EmptyDB.mdb";
        final String dbFileName = "Bank3Tier.mdb";
        try{_copyFile(emptyDbFileName, dbFileName);
        }catch(Exception ex){
            throw new multex.Failure(
            "Failure creating empty data base {1} from source file {0}",
            ex, emptyDbFileName, dbFileName
            );
        }
    }//createEmptyDB
    */

    /**Creates the application specific database tables*/
    private void _createTablesBySql(final String i_oidSuffix) {
        _pool.executeSql("create table Kunde ("
            +" oid INTEGER NOT NULL PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",name VARCHAR(50) NOT NULL"
            +",geburt DATE NOT NULL"
            +" )"
        );
        _pool.executeSql("create table Konto ("
            +" oid INTEGER NOT NULL PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",saldo INTEGER NOT NULL"
            +",eroeffnet DATE NOT NULL"
            +",geschlossen DATE"
            +" )"
        );
        _pool.executeSql("create table Verwaltet ("
            +" oid INTEGER NOT NULL PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",kunde" + i_oidSuffix + " INTEGER NOT NULL"
            +",konto" + i_oidSuffix + " INTEGER NOT NULL"
            +",inhaber BOOLEAN NOT NULL"
            +" )"
        );
    }
    
    /**Registers the class i_class for having persistent objects.*/
    public void persistClass(final Class i_class) {
        final String className = i_class.getName();
        final String localClassName = StringUtil.localClassName(className);
		final String tableName
        = localClassName.substring(2).toLowerCase();//without "Lg"
        _pool.createJoraTable(className, tableName);        
    }
    
    protected void dbDeleteTables() throws multex.Failure {
        try{
            final jora.Session session = _getSession();
            session.closePreparedStatements();
            _deleteTables();
            dbEndTransaction(true);
            session.close();
        }catch(Exception ex){
            throw new multex.Failure("Failure deleting the application "
                + "specific database tables", ex
            );
        }
    }

	/**@deprecated*/
    public jora.Table getJoraTable(final Class i_class) throws NoPersistentClassExc {
		return _pool.getJoraTable(i_class);
	}

    /**Deletes the application specific database tables, if they exist.*/
    private void _deleteTables() {
        _pool.executeSql("drop table Kunde if exists");
        _pool.executeSql("drop table Konto if exists");
        _pool.executeSql("drop table Verwaltet if exists");
    }
    
    /**Gets the Pool interface for this persistence layer*/
    public Pool getPool() {
        return _pool;
    }
    

    /** Mark the beginning of an transaction.
      Unconfirms all modifications made to the database after the last
      call to {@link #endTransaction(boolean)}.
     */
    public void dbBeginTransaction() { _pool.getSession().rollback(); }

    /** Confirm or unconfirm the modifications made to the database in the
      actual transaction (since the last commit) and close this transaction.
      @param i_ok true if the transaction succeeded, then the modifications
        will be confirmed, otherwise unconfirmed.
     */
    public void dbEndTransaction(final boolean i_ok) {
        final jora.Session session = _pool.getSession();
        if(i_ok){session.commit();}else{session.rollback();}
    }

	/**Copies the given in file onto the given out file in binary mode
	private static void _copyFile(final String i_inFileName, final String i_outFileName)
	throws  java.io.IOException
	{
	    final java.io.FileInputStream  in  = new java.io.FileInputStream (i_inFileName);
	    final java.io.FileOutputStream out = new java.io.FileOutputStream(i_outFileName);
	    final byte[] buffer = new byte[1024];
	    for(;;){
	        final int len = in.read(buffer);
	        if(len<=0){break;}
			out.write(buffer, 0, len);
	    }//for
	}//_copyFile
	*/

    

}