package bank3tier.db; //_JoraPool.java

//2005-04-07  Knabe  Erstellt


import bank3tier.db.DbObject.OwnerPoolChangedExc;
import multex.Failure;
import jora.Cursor.NextFailure;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;



/**Implementation of the persistence layer. Holds one database session.
 * @author knabe
 */
/*package*/ class _JoraPool implements Pool, PoolAdmin {

    /**Opens an application specific JORA database.*/
    public _JoraPool(){
        try {
			//Instead of the following line you could create a JORA session
			//using another driver than JDBC-ODBC-bridge:
			//_session = new jora.Session(sun.jdbc.odbc.JdbcOdbcDriver.class);
			_session = new jora.Session(org.hsqldb.jdbcDriver.class);
			
			//_session.open("jdbc:odbc:Bank3Tier", "", "");
			_session.open("jdbc:hsqldb:mem:bank3tier", "sa", "");
			_session.connection.setAutoCommit(false);
		} catch (final Exception ex) {
            throw new Failure("Could not create persistence", ex);
		}
    }
    
    public void close(){
        _session.close();
        _session = null;
    }
    
    //implements from Pool
    public boolean save(final DbObject io_object) throws ConcurrentUpdateExc, DbObject.OwnerPoolChangedExc {
        if(io_object.isPersistent()){
            //make update
            _update(io_object);
            return false;
        }
    	//make insert
        io_object.oid = _getNewOid();
        io_object.initPool(this);
		final jora.Table table = getJoraTable(io_object);
		table.insert(io_object);
		return true;
    }

    /** Update the representation in the database from this object.
     * @return number of objects actually updated
     */
    private int _update(final DbObject io_object) throws ConcurrentUpdateExc
    {
        final jora.Table table = getJoraTable(io_object);
        try{
            final DbObject dbObject = _selectByOid(table, io_object.getOid());
            //old approach: table.copy(dbObject, this);
            //new approach:
            if(dbObject.db__updateVersion!=io_object.db__updateVersion){ //meanwhile updated in the database
                throw new ConcurrentUpdateExc(table.name(), io_object.getOid());
            }

            io_object.db__updateVersion++;
            return table.update(io_object);
        } catch(ConcurrentUpdateExc ex){throw ex;
        } catch(Exception ex){
            throw new multex.Failure(
                "Failure updating object with OID {0} in table {1}",
                ex, io_object.getOid(), table.name()
            );
        }
    }

    //implements from Pool
    public final boolean delete(final DbObject io_object)
    { final int rowCount = getJoraTable(io_object).delete(io_object); io_object.oid = Pool.deletedOid; return rowCount>=1;}

    public DbObject find(final Class i_persistentClass, final Integer i_oid) 
    throws NoPersistentClassExc, Pool.OidNotFoundExc {
        final jora.Table table = getJoraTable(i_persistentClass);
        return _selectByOid(table, i_oid);
    }
    
    /** Returns the object with the OID i_oid from the database table i_table.
     *  The returned object will be dynamically an instance of the type given
     *  as first parameter to the constructor of i_table.
     *  Note: The selected object should
     *  have no components of InputStream, Blob or Clob type, because
     *  their data will be not available after closing the Cursor used to get
     *  it.
     *  @throws Pool.OidNotFoundExc If in i_table no object with OID i_oid was found.
     */
    private static DbObject _selectByOid(final jora.Table i_table, final Integer i_oid) throws Pool.OidNotFoundExc
    {
        final jora.Cursor c = i_table.selectAll("where oid="+i_oid);
        final Object result = c.next();
        if(result==null){
            throw new Pool.OidNotFoundExc(i_oid, i_table);
        }
        //???Wegen Verlust der BLOBS usw. so sinnvoll?
        c.close();
        return (DbObject)result;
    }

    //implements from Pool
    public List findAll(final Class i_persistentClass) throws NoPersistentClassExc {
        _checkPersistentClass(i_persistentClass);
        try {
            final jora.Cursor c = _getJoraTableUnchecked(i_persistentClass).select("");
            return _copyToList(c);
        }catch(Exception ex){throw new multex.Failure(
            "Failure searching all objects of class {0}", ex, i_persistentClass.getName());
        }
    }

    //implements from Pool
    public DbObject findUniqueByExample(final DbObject i_object) throws ExampleNotFoundExc, ExampleNotUniqueExc
    {
        try {
			final Object[] allObjects = _findByExample(i_object);			
            if(allObjects.length>1){throw new ExampleNotUniqueExc(i_object);
			}else if(allObjects.length<1){throw new ExampleNotFoundExc(i_object);
			}
			final DbObject result = (DbObject)allObjects[0];
			result.initPool(this);
			return result;
        } catch (ExampleNotFoundExc e){throw e;
        } catch (ExampleNotUniqueExc e){throw e;
		} catch (Exception e) {
			throw new Failure("Failure finding unique by example {0}", e, i_object);
		}
    }

    //implements from Pool
    public /*DbObject*/List findManyByExample(final DbObject i_object)
    {
        try {
            final Object[] allObjects = _findByExample(i_object);
            for(int i=0; i<allObjects.length; i++){
                final DbObject dbObject = (DbObject)allObjects[i];
                dbObject.initPool(this);
            }
            return Arrays.asList(allObjects);
        } catch (Exception e) {
            throw new Failure("Failure finding objects by example {0}", e, i_object);
        }
    }

    /**
	 * @param i_object
	 * @return
	 */
	private Object[] _findByExample(final DbObject i_object) {
		final jora.Table table = getJoraTable(i_object);
		final jora.Cursor cursor = table.queryByExample(i_object);
		final Object[] allObjects = cursor.toArray();
		final String query = cursor.getQuery(); //Query only known after Cursor.next()

		return allObjects;
	}

	//implements from Pool
    public boolean areLinked(final DbObject i_left, final Class i_linkClass, final DbObject i_right) throws NoPersistentClassExc {
        final jora.Table table = getJoraTable(i_linkClass);
		final Object[] links = table.select(
            "WHERE " + _linkCondition(i_left)
            + " AND "+ _linkCondition(i_right)
        ).toArray();
        return links.length >= 1;
    }

    //implements for Pool
    public DbObject getLink(final DbObject i_object, final Class i_partnerClass) throws NoPersistentClassExc, Pool.LinkFailure
    {
        final Class ownClass = i_object.getClass();
        String partnerTableName = "";
        String partnerOidFieldName = "";
        Integer partnerOid = null;
        try {
            //Get the Table of i_partnerClass:
            final jora.Table partnerTable = getJoraTable(i_partnerClass);;
            
            //Determine the OID of the linked partner object:
            partnerTableName = partnerTable.name();
            partnerOidFieldName = partnerTableName.toLowerCase() + DbObject.oidSuffix;
            final java.lang.reflect.Field partnerOidField
            = ownClass.getDeclaredField(partnerOidFieldName);
            partnerOidField.setAccessible(true);
            final Object partnerOidObject = partnerOidField.get(i_object);
            partnerOid = (Integer)partnerOidObject;

            //Get the object of the partner table with the partner OID:
            return _selectByOid(partnerTable, partnerOid);
        } catch(final NoPersistentClassExc ex){throw ex;
        } catch(final Exception ex){
            throw new Pool.LinkFailure(ex, ownClass.getName(), partnerOidFieldName, partnerTableName, partnerOid);
        }
    }

    //implements for Pool
    public /*DbObject*/List findManyByQuery(
        final Class i_resultClass, 
        final Class i_queryClass, 
        final Object[] i_args
    ) 
    throws NoPersistentClassExc, NoQueryClassExc, ArgumentCountExc, ArgumentTypeExc
    {
        _checkPersistentClass(i_resultClass); //throws NoPersistentClassExc
        if(!Query.class.isAssignableFrom(i_queryClass)){
            throw new NoQueryClassExc(i_queryClass, Query.class);
        }
        try {
            final Query query = (Query)i_queryClass.newInstance();
            final jora.Table table = _getJoraTableUnchecked(i_resultClass);
            final jora.Cursor cursor = query.execute(this, table, i_args);
            return _copyToList(cursor);
        }catch(ArgumentCountExc ex){throw ex;
        }catch(ArgumentTypeExc ex){throw ex;
        }catch(Exception ex){throw new multex.Failure(
            "Failure searching all objects of class {0}", ex, i_resultClass.getName());
        }
    }
    
    /**
	 * @param io_cursor
	 * @return
	 * @throws NextFailure
	 * @throws OwnerPoolChangedExc
	 */
	private List _copyToList(final jora.Cursor io_cursor) throws NextFailure, OwnerPoolChangedExc {
		final List result = new ArrayList();
		for(;;){
		    final DbObject o = (DbObject)io_cursor.next();
		    if(o==null){break;}
		    o.initPool(this);
		    result.add(o);
		}
		return result;
	}

	/**Returns the SQL condition, that i_linkPartner's OID is the same as its table name followed by the OID suffix.
     * @throws NoPersistentClassExc*/
	private String _linkCondition(final DbObject i_linkPartner){
        final jora.Table table = getJoraTable(i_linkPartner);
		final String linkTableName = table.name();
		return linkTableName + DbObject.oidSuffix + " = " + i_linkPartner.getOid();
	}

	/**Creates a JORA Table with the name i_tableName for storing objects of class i_className.*/
    /*package*/ void createJoraTable(final String i_className, final String i_tableName) {
        final jora.Table table = new jora.Table(
            i_className, i_tableName, getSession(), "oid"
        );
        tables.put(i_className, table);
    }

	private void _checkPersistentClass(final Class i_persistentClass) throws NoPersistentClassExc {
		if(!DbObject.class.isAssignableFrom(i_persistentClass)){
            throw new NoPersistentClassExc(i_persistentClass);
        }
	}

	/*package*/void executeSql(final String i_sql){
        _session.execute(i_sql);
    }

    /*package*/jora.Session getSession(){return _session;}

    private static int _lastOid = Pool.deletedOid.intValue();

    /** Returns an object identification number unique for each object in our
      * application. In a non-demo application with real persistency the
      * _lastOid should be stored in the database, too, rather than in a
      * variable.
    */
    private Integer _getNewOid(){return new Integer(++_lastOid);}

	private jora.Session _session;
	/**
	 * Maps each class name to its JORA Table
	 */
	/*package*/ Map tables = new HashMap();

    /**Returns the JORA Table, in which objects of i_persistentClass are stored.*/
    /*package*/ jora.Table getJoraTable(final Class i_persistentClass) throws Pool.NoPersistentClassExc {
        _checkPersistentClass(i_persistentClass);
        return _getJoraTableUnchecked(i_persistentClass);
    }

    /**Returns the JORA Table, in which i_persistentObject should be stored.*/
    /*package*/ jora.Table getJoraTable(final DbObject i_persistentObject){
        return _getJoraTableUnchecked(i_persistentObject.getClass());
    }

    /**Returns the JORA Table, in which objects of i_persistentClass are stored.*/
	private jora.Table _getJoraTableUnchecked(final Class i_persistentClass) throws Failure {
		final String className =i_persistentClass.getName(); 
	    if(!tables.containsKey(className)){
	        throw new Failure("Class {0} was not registered for JORA persistence", null, className);
	    }
	    return (jora.Table)tables.get(className);
	}
}
