package bank3tier.lg; //LgApplication.java

import java.sql.Date;
import java.util.List;

import bank3tier.db.DbApplication;
import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.DbReicheKundenQuery;
import bank3tier.db.Pool;

/**Steht fr die Applikation als Ganzes. Enthlt eine Operation zur 
  Initialisierung der Applikation sowie statische Operationen zum Erfassen 
  einiger Startobjekte, von denen ausgehend weitere Objekte mit Hilfe von
  Objektoperationen erfasst werden.
 * @author  Christoph Knabe
 */
public class LgApplication {

    /**Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden*/
	public static class KeinKundeExc extends multex.Exc {
	    public KeinKundeExc(final LgDatum i_geburtDate){
	        super("Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden", i_geburtDate.toDate());
	    }
	}

	private final DbApplication _dbApplication;
    private final Pool _pool;
    
    /** Initializes the application.*/
    public LgApplication(final DbApplication io_dbApplication) throws multex.Failure {
        try {
            _dbApplication = io_dbApplication;
            _dbApplication.persistClass(LgKunde.class);
            _dbApplication.persistClass(LgKonto.class);
            _dbApplication.persistClass(LgVerwaltet.class);
            _pool = _dbApplication.getPool();
        } catch(final Exception ex){
            throw new multex.Failure("Error initializing the application", ex); 
        }
    }
    
    /**Returns the persistence Pool, this application is operating on*/
    /*package*/ Pool getPool(){return _pool;}
    
    /**Beginnt eine Applikationstransaktion.*/
    public void beginTransaction(){
        _dbApplication.dbBeginTransaction();
    }
    
    /**Beendet eine Applikationstransaktion. Besttigt nderungen, wenn i_ok, verwirft andernfalls.*/
    public void endTransaction(final boolean i_ok){
        _dbApplication.dbEndTransaction(i_ok);
    }
    
    /**Erzeugt ein neues Kundenobjekt als externe, geprfte Operation.*/    
    public LgKunde kundeErfassen(final String i_name, final LgDatum i_geburt){
        try {
            final LgKunde kunde = new LgKunde();
            kunde.setzen(i_name, i_geburt);
            _pool.save(kunde);
            return kunde;
        }catch(final Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu erfassen", ex, i_name);
        }
    }

    /**Liefert alle Kunden*/
    public LgKunde_List alleKundenSuchen(){
        try {
            final List list = _pool.findAll(LgKunde.class);
            final LgKunde_List result = new LgKunde_List(list);
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Failure searching all LgKunde-objects", ex);
        }
    }

    /**Liefert alle Kunden, die ab dem bergebenen Datum geboren wurden.
     * Suche in einer Tabelle, Abfrage nur eigener Attribute.*/
    public LgKunde_List jungeKundenSuchen(final LgDatum i_datum){
        final Date date = i_datum.toDate();
        try {
            final List list = _pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{date});
            final LgKunde_List result = new LgKunde_List(list);
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Fehler beim Suchen von Kunden, die ab {0,date} geboren wurden", ex, date);
        }
    }
            
    /**Liefert den Kunden mit dem angegebenen Geburtsdatum*/
    public LgKunde kundenSuchen(final LgDatum i_geburtsDatum)
    throws LgApplication.KeinKundeExc, Pool.ExampleNotUniqueExc
    {
        try {
            final LgKunde example = new LgKunde();
            example.setGeburt(i_geburtsDatum);
            final LgKunde result = (LgKunde)_pool.findUniqueByExample(example);
            return result;
        }catch(final Pool.ExampleNotFoundExc ex){
            final KeinKundeExc keinKundeExc = new LgApplication.KeinKundeExc(i_geburtsDatum);
            keinKundeExc.initCause(ex);
			throw keinKundeExc;
        }
    }

    /**TODO Implement: Liefert alle Kunden, die Inhaber eines Kontos sind, das einen Saldo grer oder gleich 
     * i_saldoCt aufweist.
     * */
    public LgKunde_List reicheKundenSuchen(final int i_saldoCt){
        try {
            final List list = _pool.findManyByQuery(
                LgKonto.class, DbReicheKundenQuery.class, new Integer[]{new Integer(i_saldoCt)}
            );
            final LgKunde_List result = new LgKunde_List(list);
            return result;
        }catch(final Exception ex){throw new multex.Failure(
            "Fehler beim Suchen von Kunden, die ab {0} Cent auf einem Konto haben", ex, new Integer(i_saldoCt)
        );}
    }

}