/*
 * LgKonto.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:44
 */

package bank3tier.lg;
import  bank3tier.db.DbObject;
import bank3tier.db.Pool;

/** * Ein Konto bei einer Bank.
 * @author  Christoph Knabe
 */
public class LgKonto extends DbObject {
    //Attribute:
    private int saldo = 0;
    private java.sql.Date eroeffnet
    = new java.sql.Date(new java.util.Date().getTime());
    private java.sql.Date geschlossen = null;

    /*package*/ LgKonto(){}
    /*package*/ LgKonto(final Pool io_pool){super(io_pool);}
    
    /**Liefert den aktuellen Kontosaldo in Cent.*/
    public int getSaldo(){return saldo;}

    /**Der Mindestkontosaldo, der durch Verfgungen ber das Konto nicht unterschritten werden darf.*/
    public static final int mindestSaldo = -1000000;

    /**Trgt den Kunden als weiteren Verwalter dieses Kontos ein.*/
    public void verwalterHinzufuegen(final LgKunde i_kunde)throws SchonVerwalterExc{
        final Pool pool = getPool();
        try {
            if(pool.areLinked(i_kunde, LgVerwaltet.class, this)){
                throw new SchonVerwalterExc(i_kunde, this);
            }
            final LgVerwaltet verwaltet = new LgVerwaltet(i_kunde, this);
            verwaltet.setInhaber(false); //kein Inhaber
            pool.save(verwaltet);
        }catch(final SchonVerwalterExc ex){throw ex;
        }catch(final Exception ex){
            throw new multex.Failure(
                "Verwalter konnte nicht hinzugefgt werden.", ex
            );
        }
    }

    /**berweist den Geldbetrag i_betrag in Cent von diesem Konto auf das
     * Konto io_zielKonto.
    */
    public void ueberweisen(
        final LgKonto io_zielKonto, final int i_betrag,
        final LgKunde i_auftraggeber
    ) throws OhneRechteExc, MindestSaldoExc
    {

        try {
            //1. Fehlerprfung:
            final Pool pool = getPool();
            if(!pool.areLinked(i_auftraggeber, LgVerwaltet.class, this))
            {throw new OhneRechteExc(i_auftraggeber, this);}
            final int neuSaldo = saldo - i_betrag;
            if(neuSaldo < mindestSaldo){throw new MindestSaldoExc(neuSaldo, mindestSaldo);}

            //2. Vernderungen im Arbeitsspeicher vornehmen:
            this.saldo = neuSaldo;
            io_zielKonto.saldo += i_betrag;
            
            //3. Vernderungen in Datenbank sichern:
			pool.save(this);
            pool.save(io_zielKonto);
        }catch(OhneRechteExc ex){throw ex;
        }catch(MindestSaldoExc ex){throw ex;
        }catch(final Exception ex){throw new multex.Failure(
            "Betrag {0} Cent konnte nicht berwiesen werden", ex, new Long(i_betrag)
            );
        }
    }

    /**Kunde ist kein berechtigter Verwalter des Kontos.*/
    public static class OhneRechteExc extends multex.Exc {
        public OhneRechteExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde mit OID {0} ist kein berechtigter Verwalter des Kontos mit OID {1}",
                i_kunde.getOid(), i_konto.getOid()
            );
        }
    }

    /**Mindestsaldo {0} Cent wrde unterschritten werden.*/
    public static class MindestSaldoExc extends multex.Exc {
        public MindestSaldoExc(final int i_neuSaldo, final int i_mindestSaldo){
            super("Neuer Saldo {0} Cent wrde Mindestsaldo {1} Cent unterschreiten",
                new Integer(i_neuSaldo), new Integer(i_mindestSaldo)
            );
        }
    }

    public static class SchonVerwalterExc extends multex.Exc {
        public SchonVerwalterExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde mit OID {0} ist schon Verwalter von Konto mit OID {1}",
                i_kunde.getOid(), i_konto.getOid()
            );
        }
    }

}