/*
 * LgKunde.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:34
 */

package bank3tier.lg;
import java.util.Iterator;
import java.util.List;

import  bank3tier.db.DbObject;
import bank3tier.db.Pool;
import bank3tier.ut.StringUtil;
/**
 * Ein Kunde einer Bank.
 * @author  Christoph Knabe
 */
public class LgKunde extends DbObject {
    private String name;
    private java.sql.Date geburt;
    //private LgDatum geburt; //geht nicht, obwohl von java.sql.Date abgeleitet

    public void setName(final String i_name){name = i_name;}
    public String getName(){return name;}

    public void setGeburt(final LgDatum i_geburt){geburt = i_geburt.toDate();}
    public LgDatum getGeburt(){return new LgDatum(geburt);}

    /**ndert das Kundenobjekt als externe, geprfte Operation*/
    public void aendern(){
        try {
            getPool().save(this);
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu ndern", ex, name);
        }
    }

    /**Lscht das Kundenobjekt als externe geprfte Operation.*/
    public void loeschen(){
        try {
            getPool().delete(this);
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu lschen", ex, getName());
        }
    }

    /**Erffnet ein neues Konto, dessen Inhaber der aktuelle Kunde wird.
      Als externe geprfte Operation.
      @return das neue Konto
    */
    public LgKonto kontoEroeffnen(){
        try {
            final LgKonto konto = new LgKonto();
            //konto.setPool(_pool);
            getPool().save(konto);
            final LgVerwaltet verwaltet = new LgVerwaltet(this, konto);
            verwaltet.setInhaber(true); //als Inhaber
            getPool().save(verwaltet);
            return konto;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, ein Konto zu erffnen", ex);
        }
    }

    /**Setzt diese Attribute im aktuellen Objekt.
      Als interne, unpersistente Operation.
    */
    public void setzen(final String i_name, final LgDatum i_geburt){
        name = i_name;   geburt = i_geburt.toDate();
    }

    /**Listet alle Konten des aktuellen Kunden auf mit ihrem Saldo*/
    public void printKontenStaende(){
        final Pool pool = getPool();
        try{
            //QueryByExample: Alle Verwaltet-Links dieses Kunden:
            final LgVerwaltet query = new LgVerwaltet(this);
            final /*LgVerwaltet*/List verwaltetList = pool.findManyByExample(query);
            System.out.println("-----------------------------");
            System.out.println("KontenStnde von Kunde " + name);
            System.out.println("-----------------------------");
            System.out.println("KontoOid,\tStatus,\tSaldo");
            for(final Iterator i=verwaltetList.iterator(); i.hasNext();){
                final LgVerwaltet verwaltet = (LgVerwaltet)i.next();
                final String status = verwaltet.isInhaber() ? "Inhaber" : "Verwalter";
                final LgKonto konto = (LgKonto)pool.getLink(verwaltet, LgKonto.class);
                System.out.println("#" + konto.getOid() + " " + status+ " " + konto.getSaldo());
            }
        }catch(Exception ex){throw new multex.Failure(
            "Fehler beim Ausgeben der KontenStnde for Kunde OID {0}", ex, getOid());
        }
    }

    public String toString(){
        final String localClassName = StringUtil.localClassName(getClass().getName());
        return localClassName + " {oid=" + getOid() + ", name=" + name
            + ", geburt=" + geburt + "}"
        ;
    }

}//LgKunde