package bank3tier.lg;

import java.sql.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.Assert;

import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.DbObject;
import bank3tier.db.Pool;
import bank3tier.db.DbObject.OwnerPoolChangedExc;
import bank3tier.db.Pool.ArgumentCountExc;
import bank3tier.db.Pool.ArgumentTypeExc;
import bank3tier.db.Pool.ConcurrentUpdateExc;
import bank3tier.db.Pool.ExampleNotFoundExc;
import bank3tier.db.Pool.ExampleNotUniqueExc;
import bank3tier.db.Pool.LinkFailure;
import bank3tier.db.Pool.NoPersistentClassExc;
import bank3tier.db.Pool.NoQueryClassExc;
import bank3tier.db.Pool.OidNotFoundExc;

public class TestLgKontoMock extends junit.framework.TestCase {

	public TestLgKontoMock(final String i_name) {
		super(i_name);
	}
   
    private class DummyPool implements Pool {

        //Dummy implementations for all methods of Pool:
        public boolean save(DbObject io_object) throws ConcurrentUpdateExc,
                OwnerPoolChangedExc {return false;}
        public boolean delete(DbObject io_object) {
            return false;
        }
        public DbObject find(Class i_persistentClass, Integer i_oid)
                throws NoPersistentClassExc, OidNotFoundExc {
            return null;
        }
        public List findAll(Class i_persistentClass) throws NoPersistentClassExc {
            return null;
        }
        public DbObject findUniqueByExample(DbObject i_object)
                throws ExampleNotFoundExc, ExampleNotUniqueExc {
            return null;
        }
        public List findManyByExample(DbObject i_object) {
            return null;
        }
        public boolean areLinked(DbObject i_left, Class i_linkClass,
                DbObject i_right) throws NoPersistentClassExc {
            return false;
        }
        public DbObject getLink(DbObject i_object, Class i_partnerClass)
                throws LinkFailure, NoPersistentClassExc {
            return null;
        }
        public List findManyByQuery(Class i_resultClass, Class i_queryClass,
                Object[] i_args) throws NoPersistentClassExc, NoQueryClassExc,
                ArgumentCountExc, ArgumentTypeExc {
            return null;
        }

    }
    
    public void testUeberweisen_OhneRechteExc() throws Exception {
        final Pool pool = new DummyPool();
        final LgKonto knabeKonto = new LgKonto(pool);
        final LgKunde knabe = new LgKunde();
        final LgKonto muellerKonto = new LgKonto(pool);
        try{
            knabeKonto.ueberweisen(muellerKonto, 12345, knabe);
            fail("LgKonto.OhneRechteExc expected");
        } catch ( LgKonto.OhneRechteExc expected){
        }
    }
    
    /**Eine Mock-Implementierung arbeitet aktiv am Testen mit.*/
    private class UeberweisenMockPool extends DummyPool {
        private final Set _saved = new HashSet();

        public boolean areLinked(
            DbObject i_left, Class i_linkClass, DbObject i_right
        ){
            return true;
        }
        public boolean save(DbObject io_object) 
        {
            _saved.add(io_object);
            return false;
        }
        public void checkSaved(final LgKonto i_knabeKonto, final LgKonto i_muellerKonto) throws Exception {
            assertEquals(2, _saved.size());
            assertNotSame(i_knabeKonto, i_muellerKonto);
            assertTrue(_saved.contains(i_knabeKonto));
            assertTrue(_saved.contains(i_muellerKonto));
        }
    }
    
    public void testUeberweisen() throws Exception {
        final UeberweisenMockPool pool = new UeberweisenMockPool();
        final LgKunde knabe = new LgKunde();
        final LgKonto knabeKonto = new LgKonto(pool);
        final LgKonto muellerKonto = new LgKonto(pool);

        knabeKonto.ueberweisen(muellerKonto, -LgKonto.mindestSaldo, knabe);

        assertEquals(+LgKonto.mindestSaldo, knabeKonto.getSaldo());
        assertEquals(-LgKonto.mindestSaldo, muellerKonto.getSaldo());
        pool.checkSaved(knabeKonto, muellerKonto);
    }
    
    public void testUeberweisen_MindestSaldoExc() throws Exception {
        final Pool pool = new DummyPool(){
            public boolean areLinked(
                DbObject i_left, Class i_linkClass, DbObject i_right
            ){   return true;   }
        };
        final LgKonto knabeKonto = new LgKonto(pool);
        final LgKunde knabe = new LgKunde();
        final LgKonto muellerKonto = new LgKonto(pool);
        try{
            knabeKonto.ueberweisen(muellerKonto, -LgKonto.mindestSaldo + 1, knabe);
            fail("LgKonto.MindestSaldoExc expected");
        } catch ( LgKonto.MindestSaldoExc expected){
        }
    }


}//class
