package bank3tier.lg;

import java.sql.Date;
import java.util.List;

import bank3tier.db.DbApplication;
import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.Pool;
import bank3tier.db.Pool.ArgumentCountExc;
import bank3tier.db.Pool.ArgumentTypeExc;
import bank3tier.db.Pool.NoPersistentClassExc;
import bank3tier.db.Pool.NoQueryClassExc;

public class TestLgKunde extends junit.framework.TestCase {

    private static final LgApplication _lgApplication = new LgApplication(new DbApplication());
    
    /**Erwartete Inhalte der Datenbank*/
    private static LgKunde _mueller, _knabe, _skoric;
    
    

	public TestLgKunde(final String i_name) {
		super(i_name);
	}

    public void testErfassen(){
        //3 Musterkunden erfassen:
        _mueller = _erfassen("Manfred Mller", "27.02.1960");
        _knabe = _erfassen("Christoph Knabe", "31.05.1954");
        _skoric = _erfassen("Gordana Skoric-Knabe", "14.11.1962");
        assertEquals(new Integer(2), _knabe.getOid());
        assertEquals("Christoph Knabe", _knabe.getName());
        assertEquals("31.05.1954", _knabe.getGeburt().toString());
    }

    public void testEinzelnenSuchen()
    throws Exception
    {
        final LgKunde knabe
        = _lgApplication.kundenSuchen(LgDatum.fromString("31.05.1954"));
        assertEquals(_knabe.toString(), knabe.toString());

        try{_lgApplication.kundenSuchen(LgDatum.fromString("01.01.1970"));
        	fail("LgApplication.KeinKundeExc expected");
        } catch ( LgApplication.KeinKundeExc expected){}
    }

    public void testAlleSuchen(){
        final LgKunde_List kunden = _lgApplication.alleKundenSuchen();
		assertEquals(3, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde knabe   = kunden.get(1);
        assertEquals(_knabe.toString(), knabe.toString());

        final LgKunde skoric  = kunden.get(2);
        assertEquals(_skoric.toString(), skoric.toString());
    }

    public void testAendern()
    throws Exception
    {
        
        final String knabeGeburt = "31.05.1954";
		final LgKunde knabeAlt
        = _lgApplication.kundenSuchen(LgDatum.fromString(knabeGeburt));
        assertEquals(_knabe.toString(), knabeAlt.toString());

        knabeAlt.setName("Prof. Knabe");
        final String geaenderteGeburt = "31.12.2000";
		knabeAlt.setGeburt(LgDatum.fromString(geaenderteGeburt));
        knabeAlt.aendern();
		_knabe = knabeAlt;

        final LgKunde knabeNeu
        = _lgApplication.kundenSuchen(LgDatum.fromString(geaenderteGeburt));
        assertEquals(_knabe.toString(), knabeNeu.toString());
        
        //Sollzustand wiederherstellen:
        knabeNeu.setGeburt(LgDatum.fromString(knabeGeburt));
        knabeNeu.aendern();

        final LgKunde knabeNeuNeu
        = _lgApplication.kundenSuchen(LgDatum.fromString(knabeGeburt));
        assertEquals(knabeNeu.toString(), knabeNeuNeu.toString());
    }

    public void testJungeKundenSuchen() throws Exception {
        final LgKunde_List kunden = _lgApplication.jungeKundenSuchen(LgDatum.fromString("01.01.1960"));
        assertEquals(2, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde skoric  = kunden.get(1);
        assertEquals(_skoric.toString(), skoric.toString());
    }

    public void testReicheKundenSuchen() throws Exception {
        //TODO Fertigstellen. Knabe 2005-04-22
        final int reichSaldoCent = 1000*100;
        final LgKunde_List kunden = _lgApplication.reicheKundenSuchen(reichSaldoCent);
        assertEquals(2, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde skoric  = kunden.get(1);
        assertEquals(_skoric.toString(), skoric.toString());
    }

    /**Tests universal pool query for exceptions 
     * NoPersistentClassExc, NoQueryClassExc, ArgumentCountExc, ArgumentTypeExc
    */
    public void testQueryExceptions() throws Exception {
    	final Pool pool = _lgApplication.getPool();
        final Date date = LgDatum.fromString("01.01.1960").toDate();
        try {
            pool.findManyByQuery(DbJungeKundenQuery.class, DbJungeKundenQuery.class, new java.sql.Date[]{date});
            fail("NoPersistentClassExc expected");
        } catch ( NoPersistentClassExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, LgKunde.class, new java.sql.Date[]{date});
            fail("NoQueryClassExc expected");
        } catch ( NoQueryClassExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{});
            fail("ArgumentCountExc expected");
        } catch ( ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{date,date});
            fail("ArgumentCountExc expected");
        } catch ( ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new Object[]{"hallo"});
            fail("ArgumentTypeExc expected");
        } catch ( ArgumentTypeExc expected){}
    }

    public void testAlleLoeschen(){
        //Alle Kunden wieder lschen:
        final LgKunde_List kundenAlt = _lgApplication.alleKundenSuchen();
        for(final LgKunde_List.Iterator i=kundenAlt.listIterator(); i.hasNext();){
			final LgKunde kunde = i.next();
            kunde.loeschen();
            assertTrue(kunde.isDeleted());
        }
        _mueller = _knabe = _skoric = null;

        //Jetzt mu die Tabelle wieder leer sein:
        final LgKunde_List kundenNeu = _lgApplication.alleKundenSuchen();
		assertEquals(0, kundenNeu.size());

    }

    /**Erfasst einen Kunden mit diesem Namen und diesem Geburtsdatum persistent.
     * @param i_geburt Geburtsdatum in lokaler Schreibweise
    */
    private LgKunde _erfassen(final String i_name, final String i_geburt){
        return _lgApplication.kundeErfassen(i_name, LgDatum.fromString(i_geburt));
    }


}//class
