//-< FieldDescriptor.java >------------------------------------------*--------*
// JORA                       Version 2.0        (c) 1998  GARRET    *     ?  *
// (Java Object Relational Adapter)                                  *   /\|  *
//                                                                   *  /  \  *
//                          Created:     10-Jun-98    K.A. Knizhnik  * / [] \ *
//                          Last update: 25-Jun-98    K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Table field descriptor
//-------------------------------------------------------------------*--------*

package jora;

import java.sql.*;
import java.math.*;
import java.lang.reflect.*;

// * @version 2002-06-24 Added checks for SQLWarning

class FieldDescriptor {
    protected int    inType;  // type tag for field input (see constants below)
    protected int    outType; // type tag for field output
    protected int    scale;   // scale for tDecimal type,
    protected String name;    // full (compound) name of component
    protected Field  field;   // field info from java.lang.reflect

    protected Constructor constructor; // constructor of object component

    protected static final int t_byte         = 0;
    protected static final int t_short        = 1;
    protected static final int t_int          = 2;
    protected static final int t_long         = 3;
    protected static final int t_float        = 4;
    protected static final int t_double       = 5;
    protected static final int t_boolean      = 6;
    protected static final int tByte          = 7;
    protected static final int tShort         = 8;
    protected static final int tInteger       = 9;
    protected static final int tLong          = 10;
    protected static final int tFloat         = 11;
    protected static final int tDouble        = 12;
    protected static final int tBoolean       = 13;
    protected static final int tDecimal       = 14;
    protected static final int tString        = 15;
    protected static final int tBytes         = 16;
    protected static final int tDate          = 17;
    protected static final int tTime          = 18;
    protected static final int tTimestamp     = 19;
    protected static final int tStream        = 20;
    protected static final int tBlob          = 21;
    protected static final int tClob          = 22;
    protected static final int tAsString      = 23;

    protected static final int tClosure       = 24;
    protected static final int tCompound      = 25;
    //Higher values represent parts of a compound field!!! Knabe 2001-03-08

    protected static final int[] sqlTypeMapping = {
        Types.INTEGER, // t_byte
        Types.INTEGER, // t_short
        Types.INTEGER, // t_int
        Types.BIGINT,  // t_long
        Types.FLOAT,   // t_float
        Types.DOUBLE,  // t_double
        Types.BIT,     // t_boolean
        Types.INTEGER, // tByte
        Types.INTEGER, // tShort
        Types.INTEGER, // tInteger
        Types.BIGINT,  // tLong
        Types.FLOAT,   // tFloat
        Types.DOUBLE,  // tDouble
        Types.BIT,     // tBoolean
        Types.NUMERIC, // tDecimal
        Types.VARCHAR,  // tString
        Types.VARBINARY,// tBytes
        Types.DATE,     // tDate
        Types.TIME,      // tTime
        Types.TIMESTAMP, // tTimestamp
        Types.LONGVARBINARY, // tStream
        Types.LONGVARBINARY, // tBlob
        Types.LONGVARCHAR,   // tClob
        Types.VARCHAR,       // tAsString
        Types.LONGVARBINARY  // tClosure
    };

    protected FieldDescriptor(Field field, String name) {
        this.name = name;
        this.field = field;
        this.scale = -1;
    }

protected final boolean isAtomic() { return inType < tClosure; }

protected final boolean isCompound() { return inType >= tCompound; }

protected final boolean isBuiltin() { return inType <= t_boolean; }


    protected final boolean bindVariable(DiagnosticPreparedStatement pstmt,
    Object obj, int column)
    throws SQLException, IllegalAccessException, java.io.IOException
    {
        try {
            switch (outType) {
                case t_byte:
                pstmt.setByte(column, field.getByte(obj));
                break;
                case t_short:
                pstmt.setShort(column, field.getShort(obj));
                break;
                case t_int:
                pstmt.setInt(column, field.getInt(obj));
                break;
                case t_long:
                pstmt.setLong(column, field.getLong(obj));
                break;
                case t_float:
                pstmt.setFloat(column, field.getFloat(obj));
                break;
                case t_double:
                pstmt.setDouble(column, field.getDouble(obj));
                break;
                case t_boolean:
                pstmt.setBoolean(column, field.getBoolean(obj));
                break;
                case tByte:
                pstmt.setByte(column, ((Byte)field.get(obj)).byteValue());
                break;
                case tShort:
                pstmt.setShort(column, ((Short)field.get(obj)).shortValue());
                break;
                case tInteger:
                pstmt.setInt(column, ((Integer)field.get(obj)).intValue());
                break;
                case tLong:
                pstmt.setLong(column, ((Long)field.get(obj)).longValue());
                break;
                case tFloat:
                pstmt.setFloat(column, ((Float)field.get(obj)).floatValue());
                break;
                case tDouble:
                pstmt.setDouble(column,((Double)field.get(obj)).doubleValue());
                break;
                case tBoolean:
                pstmt.setBoolean(column,
                ((Boolean)field.get(obj)).booleanValue());
                break;
                case tDecimal:
                pstmt.setBigDecimal(column, (BigDecimal)field.get(obj));
                break;
                case tString:
                pstmt.setString(column, (String)field.get(obj));
                break;
                case tBytes:
                pstmt.setBytes(column, (byte[])field.get(obj));
                break;
                case tDate:
                pstmt.setDate(column, (java.sql.Date)field.get(obj));
                break;
                case tTime:
                pstmt.setTime(column, (java.sql.Time)field.get(obj));
                break;
                case tTimestamp:
                pstmt.setTimestamp(column, (java.sql.Timestamp)field.get(obj));
                break;
                case tStream:
                java.io.InputStream in = (java.io.InputStream)field.get(obj);
                pstmt.setBinaryStream(column, in, in.available());
                break;
                case tBlob:
                pstmt.setBlob(column, (Blob)field.get(obj));
                break;
                case tClob:
                pstmt.setClob(column, (Clob)field.get(obj));
                break;
                case tAsString:
                pstmt.setString(column, field.get(obj).toString());
                break;
                case tClosure:
                // There is no reason to use piped streams because
                // we need to pass total number of bytes to JDBC driver
                java.io.ByteArrayOutputStream out =
                new java.io.ByteArrayOutputStream();
                java.io.ObjectOutputStream clu =
                new java.io.ObjectOutputStream(out);
                clu.writeObject(field.get(obj));
                clu.close();
                pstmt.setBytes(column, out.toByteArray());
                break;
                default:
                return false;
            }
        } catch(SQLException ex) {
            if (outType != tClosure && outType != tAsString) {
                outType = tAsString;
                System.err.println("JORA-Warning: Unable to bind field " + name + "="
                    + field.get(obj) + " of Java type " + field.getType().getName()
                    + " to SQL parameter " + column + ", SQL-string:\n"
                    + pstmt.getSql() + "\nNow giving cause stack trace, later retrying as type String:"
                );
                ex.printStackTrace();
                return bindVariable(pstmt, obj, column);
            } else {
                throw ex;
            }
        }
        return true;
    }

    protected final boolean updateVariable(ResultSet resultSet,
    Object obj, int column)
    throws SQLException, IllegalAccessException, java.io.IOException
    {
        try {
            switch (outType) {
                case t_byte:
                resultSet.updateByte(column, field.getByte(obj));
                break;
                case t_short:
                resultSet.updateShort(column, field.getShort(obj));
                break;
                case t_int:
                resultSet.updateInt(column, field.getInt(obj));
                break;
                case t_long:
                resultSet.updateLong(column, field.getLong(obj));
                break;
                case t_float:
                resultSet.updateFloat(column, field.getFloat(obj));
                break;
                case t_double:
                resultSet.updateDouble(column, field.getDouble(obj));
                break;
                case t_boolean:
                resultSet.updateBoolean(column, field.getBoolean(obj));
                break;
                case tByte:
                resultSet.updateByte(column, ((Byte)field.get(obj)).byteValue());
                break;
                case tShort:
                resultSet.updateShort(column,
                ((Short)field.get(obj)).shortValue());
                break;
                case tInteger:
                resultSet.updateInt(column, ((Integer)field.get(obj)).intValue());
                break;
                case tLong:
                resultSet.updateLong(column, ((Long)field.get(obj)).longValue());
                break;
                case tFloat:
                resultSet.updateFloat(column,
                ((Float)field.get(obj)).floatValue());
                break;
                case tDouble:
                resultSet.updateDouble(column,
                ((Double)field.get(obj)).doubleValue());
                break;
                case tBoolean:
                resultSet.updateBoolean(column,
                ((Boolean)field.get(obj)).booleanValue());
                break;
                case tDecimal:
                resultSet.updateBigDecimal(column, (BigDecimal)field.get(obj));
                break;
                case tString:
                resultSet.updateString(column, (String)field.get(obj));
                break;
                case tBytes:
                resultSet.updateBytes(column, (byte[])field.get(obj));
                break;
                case tDate:
                resultSet.updateDate(column, (java.sql.Date)field.get(obj));
                break;
                case tTime:
                resultSet.updateTime(column, (java.sql.Time)field.get(obj));
                break;
                case tTimestamp:
                resultSet.updateTimestamp(column,
                (java.sql.Timestamp)field.get(obj));
                break;
                case tStream:
                java.io.InputStream in = (java.io.InputStream)field.get(obj);
                resultSet.updateBinaryStream(column, in, in.available());
                break;
                case tBlob:
                Blob blob = (Blob)field.get(obj);
                resultSet.updateBinaryStream(column,
                blob.getBinaryStream(),
                (int)blob.length());
                break;
                case tClob:
                Clob clob = (Clob)field.get(obj);
                resultSet.updateCharacterStream(column,
                clob.getCharacterStream(),
                (int)clob.length());
                break;
                case tAsString:
                resultSet.updateString(column, field.get(obj).toString());
                break;
                case tClosure:
                // There is no reason to use piped streams because
                // we need to pass total number of bytes to JDBC driver
                java.io.ByteArrayOutputStream out =
                new java.io.ByteArrayOutputStream();
                java.io.ObjectOutputStream clu =
                new java.io.ObjectOutputStream(out);
                clu.writeObject(field.get(obj));
                clu.close();
                resultSet.updateBytes(column, out.toByteArray());
                break;
                default:
                return false;
            }//switch(outType)
        } catch(SQLException ex) {
            if (outType != tClosure && outType != tAsString) {
                outType = tAsString;
                return updateVariable(resultSet, obj, column);
            } else {
                throw ex;
            }
        }
        SqlWarningFailure.check(resultSet);
        return true;
    }

    protected final boolean loadVariable(ResultSet resultSet,
    Object obj, int column)
    throws SQLException, IllegalAccessException
    , java.io.IOException, java.io.OptionalDataException
    , ClassNotFoundException
    {
        switch (inType) {
            case t_byte:
            field.setByte(obj, resultSet.getByte(column));
            break;
            case t_short:
            field.setShort(obj, resultSet.getShort(column));
            break;
            case t_int:
            field.setInt(obj, resultSet.getInt(column));
            break;
            case t_long:
            field.setLong(obj, resultSet.getLong(column));
            break;
            case t_float:
            field.setFloat(obj, resultSet.getFloat(column));
            break;
            case t_double:
            field.setDouble(obj, resultSet.getDouble(column));
            break;
            case t_boolean:
            field.setBoolean(obj, resultSet.getBoolean(column));
            break;
            case tByte:
            byte b = resultSet.getByte(column);
            field.set(obj, resultSet.wasNull() ? null : new Byte(b));
            break;
            case tShort:
            short s = resultSet.getShort(column);
            field.set(obj, resultSet.wasNull() ? null : new Short(s));
            break;
            case tInteger:
            int i = resultSet.getInt(column);
            field.set(obj, resultSet.wasNull() ? null : new Integer(i));
            break;
            case tLong:
            long l = resultSet.getLong(column);
            field.set(obj, resultSet.wasNull() ? null : new Long(l));
            break;
            case tFloat:
            float f = resultSet.getFloat(column);
            field.set(obj, resultSet.wasNull() ? null : new Float(f));
            field.setFloat(obj, resultSet.getFloat(column));
            break;
            case tDouble:
            double d = resultSet.getDouble(column);
            field.set(obj, resultSet.wasNull() ? null : new Double(d));
            break;
            case tBoolean:
            boolean bl = resultSet.getBoolean(column);
            field.set(obj, resultSet.wasNull() ? null : new Boolean(bl));
            break;
            case tDecimal:
            if (Table.useDepricatedGetBigDecimal) {
                if (scale < 0) {
                    scale = resultSet.getMetaData().getScale(column);
                }
                field.set(obj, resultSet.getBigDecimal(column, scale));
            } else {
                field.set(obj, resultSet.getBigDecimal(column));
            }
            break;
            case tString:
            field.set(obj, resultSet.getString(column));
            break;
            case tBytes:
            field.set(obj, resultSet.getBytes(column));
            break;
            case tDate:
            field.set(obj, resultSet.getDate(column));
            break;
            case tTime:
            field.set(obj, resultSet.getTime(column));
            break;
            case tTimestamp:
            field.set(obj, resultSet.getTimestamp(column));
            break;
            case tStream:
            field.set(obj, resultSet.getBinaryStream(column));
            break;
            case tBlob:
            field.set(obj, resultSet.getBlob(column));
            break;
            case tClob:
            field.set(obj, resultSet.getClob(column));
            break;
            case tClosure:
            java.io.InputStream input = resultSet.getBinaryStream(column);
            java.io.ObjectInputStream in =
            new java.io.ObjectInputStream(input);
            field.set(obj, in.readObject());
            in.close();
            break;
            default:
            return false;
        }//switch(inType)
        SqlWarningFailure.check(resultSet);
        return true;
    }

}//FieldDescriptor

