package jora; //SqlWarningFailure.java

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;

//2002-06-22  Christoph Knabe  Creation

/** The contents of a java.sql.SQLWarning wrapped into this SqlWarningFailure.
  * A JDBC operation completed leaving a SQLWarning.
  * In order not to ignore the warning, it is wrapped and then thrown.
  * This exception collects all the additional information of the SQLWarning.
*/
public class SqlWarningFailure extends Failure {
    public SqlWarningFailure(final java.sql.SQLWarning i_cause){
        super("JORA: SQLWarning with vendor-specific error code {0} and SQLState {1} encountered",
            i_cause,  new Integer(i_cause.getErrorCode()) , i_cause.getSQLState()
        );
    }

    /**Checks i_jdbcObject for a warning. Clears it afterwards.
      @throws SqlWarningFailure A SQLWarning on the JDBC object was detected
    */
    public static void check(final java.sql.Connection i_jdbcObject)
    throws SqlWarningFailure, JdbcFailure
    { _check(i_jdbcObject); }

    /**Checks i_jdbcObject for a warning. Clears it afterwards.
      @throws SqlWarningFailure A SQLWarning on the JDBC object was detected
    */
    public static void check(final java.sql.Statement i_jdbcObject)
    throws SqlWarningFailure
    { _check(i_jdbcObject); }

    /**Checks i_jdbcObject for a warning. Clears it afterwards.
      @throws SqlWarningFailure A SQLWarning on the JDBC object was detected
    */
    public static void check(final java.sql.ResultSet i_jdbcObject)
    throws SqlWarningFailure
    { _check(i_jdbcObject); }

    /**Checks i_jdbcObject, which must be of type Connection, Statement or
      ResultSet for a warning. Clears the warning afterwards.
      @throws SqlWarningFailure A SQLWarning on the JDBC object was detected
    */
    private static void _check(final Object i_jdbcObject)
    throws SqlWarningFailure {
        final java.sql.SQLWarning warning;
        try{
            if(i_jdbcObject instanceof Connection){warning = ((Connection)i_jdbcObject).getWarnings();
            }else if(i_jdbcObject instanceof Statement){warning = ((Statement)i_jdbcObject).getWarnings();
            }else{warning = ((ResultSet)i_jdbcObject).getWarnings();
            }
        }catch(java.sql.SQLException ex){throw new JdbcFailure(ex);}

        if(warning!=null){throw new SqlWarningFailure(warning);}
    }


}
